%define major     4
%define libname   %mklibname z3_ %major
%define develname %mklibname z3 -d

# Do not embed bad package note paths in the OCaml files.
%undefine _package_note_flags

# Tests are off by default because some of the tests require more memory than
# the koji builders have available.
%bcond_with test

# ocaml build underlinks
%define _disable_ld_no_undefined 1

Name:           z3
Version:        4.13.0
Release:        %mkrel 1
Summary:        Satisfiability Modulo Theories (SMT) solver
Group:          Sciences/Mathematics
License:        MIT
URL:            https://github.com/Z3Prover/z3
Source0:        https://github.com/Z3Prover/z3/archive/%{name}-%{version}.tar.gz

BuildRequires:  cmake
BuildRequires:  doxygen
BuildRequires:  gcc-c++
BuildRequires:  gmp-devel
BuildRequires:  graphviz
BuildRequires:  help2man
BuildRequires:  java-devel
BuildRequires:  javapackages-tools
BuildRequires:  make
BuildRequires:  ninja-build
BuildRequires:  ocaml
BuildRequires:  ocaml-findlib
BuildRequires:  ocaml-ocamldoc
BuildRequires:  ocaml-zarith-devel
BuildRequires:  python3-devel
BuildRequires:  python3dist(setuptools)

%description
Z3 is a satisfiability modulo theories (SMT) solver; given a set of
constraints with variables, it reports a set of values for those
variables that would meet the constraints.  The Z3 input format is an
extension of the one defined by the SMT-LIB 2.0 standard.  Z3 supports
arithmetic, fixed-size bit-vectors, extensional arrays, datatypes,
uninterpreted functions, and quantifiers.

%package -n %libname
Summary:        Library for applications that use z3 functionality
Provides:       %{name}-libs%{?_isa} = %{version}-%{release}
Provides:       z3-libs = %{version}-%{release}

%description -n %libname
Library for applications that use z3 functionality.

%package -n %develname
Summary:        Header files for build applications that use z3
Requires:       %libname = %{version}-%{release}
Provides:       z3-devel = %{version}-%{release}

%description -n %develname
Header files for build applications that use z3.

%package doc
Summary:        API documentation for Z3
License:        MIT AND GPL-3.0-or-later WITH Bison-exception-2.2 AND GPL-1.0-or-later
# FIXME: this should be noarch, but we end up with different numbers of inheritance
# graphs on different architectures.  Why?

%description doc
API documentation for Z3.

%package -n java-%{name}
Summary:        Java interface to z3
Requires:       %{name}-libs%{?_isa} = %{version}-%{release}
Requires:       java
Requires:       javapackages-tools

%description -n java-%{name}
Java interface to z3.

%package -n ocaml-%{name}
Summary:        Ocaml interface to z3
Requires:       %{name}-libs%{?_isa} = %{version}-%{release}

%description -n ocaml-%{name}
Ocaml interface to z3.

%package -n ocaml-%{name}-devel
Summary:        Files for building ocaml applications that use z3
Requires:       ocaml-%{name}%{?_isa} = %{version}-%{release}
Requires:       ocaml-zarith-devel%{?_isa}

%description -n ocaml-%{name}-devel
Files for building ocaml applications that use z3.

%package -n python3-%{name}
Summary:        Python 3 interface to z3
BuildArch:      noarch
Requires:       %{name}-libs = %{version}-%{release}

%description -n python3-%{name}
Python 3 interface to z3.

%prep
%autosetup -p1 -n %{name}-%{name}-%{version}

# Enable verbose builds, use Fedora CFLAGS, preserve timestamps when installing,
# include the entire contents of the archives in the library, link the library
# with the correct flags, and build the ocaml files with debuginfo.
sed \
  -e 's/@$(CXX)/$(CXX)/' \
  -e '/O3/d' \
  -e "s/\(['\"]\)cp\([^[:alnum:]]\)/\1cp -p\2/" \
  -e "s/\(SLIBEXTRAFLAGS = '\)'/\1-Wl,--no-whole-archive'/" \
  -e '/SLIBFLAGS/s|-shared|& %{build_ldflags} -Wl,--whole-archive|' \
  -e 's/\(libz3$(SO_EXT)\)\(\\n\)/\1 -Wl,--no-whole-archive\2/' \
  -e "s/OCAML_FLAGS = ''/OCAML_FLAGS = '-g'/" \
  -i scripts/mk_util.py

# Comply with the Java packaging guidelines and fill in the version for python
majver=$(cut -d. -f-2 <<< %{version})
sed -e '/libz3java/s,\(System\.load\)Library("\(.*\)"),\1("%{_libdir}/z3/\2.so"),' \
    -e "s/@MAJVER@/$majver/" \
    -i scripts/update_api.py

# Turn off HTML timestamps for reproducible builds
sed -i '/HTML_TIMESTAMP/s/YES/NO/' doc/z3api.cfg.in doc/z3code.dox

%build
export LANG="C.UTF-8"
export PYTHON="%{python3}"

%cmake -G Ninja \
  -DCMAKE_INSTALL_INCLUDEDIR=%{_includedir}/%{name} \
  -DGMP_CXX_INCLUDES=/usr/include \
  -DGMP_CXX_LIBRARIES=%{libdir} \
  -DCMAKE_JAVA_COMPILE_FLAGS="-source;1.8;-target;1.8" \
  -DZ3_BUILD_DOCUMENTATION:BOOL=ON \
  -DZ3_BUILD_JAVA_BINDINGS:BOOL=ON \
  -DZ3_BUILD_PYTHON_BINDINGS:BOOL=ON \
  -DZ3_INCLUDE_GIT_HASH:BOOL=OFF \
  -DZ3_INCLUDE_GIT_DESCRIBE:BOOL=OFF \
  -DZ3_USE_LIB_GMP:BOOL=ON

%cmake_build

# The cmake build system does not build the OCaml interface.  Do that manually.
#
# First, run the configure script to generate several files.
# This is NOT an autoconf-generated configure script.
./configure -p %{_prefix} --gmp --ml

# Second, to prevent make from rebuilding the entire library, copy the
# cmake-built library to where make expects it.
#cp -dp %{_vpath_builddir}/libz3.so* build

# Third, make wants to rebuild libz3.so since its dependencies do not exist.
# Do selective Makefile surgery to prevent this.
sed -i '/^api/s/ libz3\$(SO_EXT)//g' build/Makefile

# Fourth, build the OCaml interface
%make_build -C build ml

%install
export LANG="C.UTF-8"

# Install the C++, python3, and Java interfaces
%cmake_install

# Move the Java interface to its correct location
mkdir -p %{buildroot}%{_libdir}/%{name}
mkdir -p %{buildroot}%{_jnidir}
mv %{buildroot}%{_javadir}/*.jar %{buildroot}%{_jnidir}
ln -s %{_jnidir}/com.microsoft.z3.jar %{buildroot}%{_libdir}/%{name}
mv %{buildroot}%{_libdir}/lib%{name}java.so %{buildroot}%{_libdir}/%{name}

# Install the OCaml interface
pushd build/api/ml
mkdir -p %{buildroot}%{ocamldir}/Z3
cp -p META *.{a,cma,cmi,cmx,cmxa,cmxs,mli} %{buildroot}%{ocamldir}/Z3
mkdir -p %{buildroot}%{ocamldir}/stublibs
cp -p *.so %{buildroot}%{ocamldir}/stublibs
popd

# We handle the documentation files below
rm -rf %{buildroot}%{_docdir}/Z3

# Make a man page
mkdir -p %{buildroot}%{_mandir}/man1
help2man -N -o %{buildroot}%{_mandir}/man1/%{name}.1 %{_vpath_builddir}/%{name}

# Fix the pkgconfig file
sed -i 's,//usr,,' %{buildroot}%{_libdir}/pkgconfig/z3.pc

%if %{with test}
%check
export LANG="C.UTF-8"
cd build
make test-z3
./test-z3 /a
cd -
%endif

%files
%doc README.md RELEASE_NOTES.md
%{_bindir}/%{name}
%{_mandir}/man1/%{name}.1*

%files -n %libname
%license LICENSE.txt
%{_libdir}/lib%{name}.so.4*

%files -n %develname
%{_includedir}/%{name}/
%{_libdir}/lib%{name}.so
%{_libdir}/cmake/%{name}/
%{_libdir}/pkgconfig/%{name}.pc

%files doc
%doc %{_vpath_builddir}/doc/api/html examples
%license LICENSE.txt

%files -n java-%{name}
%{_libdir}/%{name}/
%{_jnidir}/com.microsoft.z3*jar

%files -n ocaml-%{name}
%dir %{ocamldir}/Z3/
%{ocamldir}/Z3/META
%{ocamldir}/Z3/*.cma
%{ocamldir}/Z3/*.cmi
%{ocamldir}/Z3/*.cmxs
%{ocamldir}/stublibs/*.so

%files -n ocaml-%{name}-devel
%{ocamldir}/Z3/*.a
%{ocamldir}/Z3/*.cmx
%{ocamldir}/Z3/*.cmxa
%{ocamldir}/Z3/*.mli

%files -n python3-%{name}
%{python3_sitelib}/%{name}/


%changelog
* Tue Aug 27 2024 neoclust <neoclust> 4.13.0-1.mga10
+ Revision: 2091579
- New version 4.13.0

* Sat Dec 30 2023 daviddavid <daviddavid> 4.12.4-1.mga10
+ Revision: 2024482
- new version: 4.12.4
- rebuild for py3.12

* Thu Feb 09 2023 daviddavid <daviddavid> 4.12.1-1.mga9
+ Revision: 1939569
- new version: 4.12.1

* Wed Jan 18 2023 daviddavid <daviddavid> 4.12.0-1.mga9
+ Revision: 1934220
- new version: 4.12.0
- drop upstreamed -data-race and -uninit patches

* Sun Jan 15 2023 daviddavid <daviddavid> 4.11.2-1.mga9
+ Revision: 1933246
- new version: 4.11.2
- add -data-race patch to fix segfault (bz 2157972)
- add -uninit patch to fix use of an uninitialized value
- use new OCaml macros

* Sun Jan 08 2023 daviddavid <daviddavid> 4.8.17-1.mga9
+ Revision: 1931575
- rebuild for ocaml-zarith change
+ neoclust <neoclust>
- New version 4.8.17

* Sat May 21 2022 pterjan <pterjan> 4.8.15-5.mga9
+ Revision: 1859659
- Rebuild for OCaml 4.14

* Sat May 14 2022 pterjan <pterjan> 4.8.15-4.mga9
+ Revision: 1858159
- Oops, I said provide

* Sat May 14 2022 pterjan <pterjan> 4.8.15-3.mga9
+ Revision: 1858158
- Provide what is required by subpackages

* Sun Apr 24 2022 joequant <joequant> 4.8.15-2.mga9
+ Revision: 1852975
- change z3 libname

* Fri Apr 22 2022 joequant <joequant> 4.8.15-1.mga9
+ Revision: 1852625
- fix groups
- imported package z3

